﻿#region usings
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;

using IdeaBlade.EntityModel;
using IdeaBlade.Core;

using DomainModel;

#endregion

namespace SilverlightPIApp {

  public partial class Page : UserControl {

    #region ctor & load

    public Page() {
      InitializeComponent();
      Loaded += Page_Loaded;
    }

    void Page_Loaded(object sender, RoutedEventArgs e) {
      InitializePersistence(); 
      InitializeQueries();
    }

    private void InitializePersistence() {
      DynamicInterceptors.AddDynamicEntityInterceptors();
      _entityManager = new NorthwindIBEntities(true);
      Login();
    }

    #endregion

    #region Button Click Handlers

    private void btnSave_Click(object sender, RoutedEventArgs e) {
      Save();
    }

    #endregion

    #region Initializequeries

    private void InitializeQueries() {
      AddQueries();
      InitializeQueryCombo();
    }

    private void InitializeQueryCombo() {
      var qNames = _queries.Keys.ToList();
      _queryCombo.ItemsSource = qNames;
      _queryCombo.SelectedItem = qNames.FirstOrDefault();
      _queryCombo.SelectionChanged += delegate { Fetch(); };
    }

    #endregion

    #region Fetch

    private void Fetch() {

      var query = GetQuery();
      if (null == query) {
        WriteMessage("No query selected; pick a query.");
        return;
      }

      WriteMessage("Fetching ...");

      _entityManager.ExecuteQueryAsync(query, FetchCallback, null);
    }

    private void FetchCallback(EntityQueryOperation args) {
      if (args.HasError) {
        WriteMessage(args.Error.Message);
        return;
      }

      // Special logic to handle the projection query - 
      //  can't bind to an anonymous type in Silverlight
      if (AnonymousFns.IsAnonymousType(args.EntityQuery.ElementType)) {
        _dataGrid.ItemsSource = DynamicTypeConverter.Convert(args.Results);
      } else {
        _dataGrid.ItemsSource = args.Results;
      }

      ReportFetchCount(args.Results);
    }

    private IEntityQuery GetQuery() {
      var queryName = (string)_queryCombo.SelectedItem;
      return queryName == null ? null : _queries[queryName];
    }

    private void ReportFetchCount(IEnumerable result) {
      WriteMessage(string.Format("Retrieved {0} customers", result.Cast<object>().Count()));
    }

    #endregion

    #region Queries 

    private void AddQueries() {

      _queries = new Dictionary<string, IEntityQuery>();

      _queries.Add("Get all Customers", _entityManager.Customers);

      _queries.Add("Get Customers starting with A",
                  _entityManager.Customers
                    .Where(c => c.CompanyName.StartsWith("A")));

      _queries.Add("Get Customers with Orders in Oregon",
                  _entityManager.Customers
                    .Where(c => c.Orders.Any(o => o.ShipRegion == "OR")));

      _queries.Add("Get Customers starting with B from cache-only",
                  _entityManager.Customers
                    .Where(c => c.CompanyName.StartsWith("B"))
                    .With(QueryStrategy.CacheOnly));

      _queries.Add("Get Customer info as projection",
                 _entityManager.Customers
                   .Select(c => new { c.CustomerID, c.CompanyName, c.City}));
    }

    #endregion

    #region Login

    private void Login() {
      var cred = new LoginCredential("demo", "demo", "demo");

      WriteMessage("Logging in ...");

      _entityManager.LoginAsync(cred, LoginCallback, null);
    }

    private void LoginCallback(LoginOperation args) {
      if (args.HasError) {
        WriteMessage(args.Error.Message);
        return;
      }

      WriteMessage("Logged in");

      _queryCombo.IsEnabled = args.IsCompleted;
      _btnSave.IsEnabled = args.IsCompleted;
      Fetch();
    }

    #endregion

    #region Save

    private void Save() {

      if (!_entityManager.HasChanges()) {
        WriteMessage("No changes to save.");
        return;
      }

      WriteMessage("Saving ...");

      _entityManager.SaveChangesAsync(SaveCallback, null);
    }

    private void SaveCallback(EntitySaveOperation args) {
      if (args.IsCompleted) {
        WriteMessage("Changes saved");
      } else {
        WriteMessage(args.Error.Message);
      }
    }

    #endregion

    #region Misc

    private void WriteMessage(string msg) {
      _txtStatus.Text = msg;
    }

    #endregion

    #region Fields

    private NorthwindIBEntities _entityManager;
    private Dictionary<string, IEntityQuery> _queries;

    #endregion
  }
}
